@@ -24,7 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include "config.h"
+#include <config.h>
 #include <stdlib.h>
 #include <sys/types.h>
 
@@ -32,33 +32,27 @@
 #include "gdkimage.h"
 #include "gdkprivate.h"
 #include "gdkinternals.h"	/* For scratch_image code */
-#include "gdkalias.h"
 
 /**
  * gdk_image_ref:
  * @image: a #GdkImage
- *
+ * 
  * Deprecated function; use g_object_ref() instead.
  * 
  * Return value: the image
- *
- * Deprecated: 2.0: Use g_object_ref() instead.
  **/
 GdkImage *
 gdk_image_ref (GdkImage *image)
 {
-  g_return_val_if_fail (GDK_IS_IMAGE (image), NULL);
-
-  return g_object_ref (image);
+  return (GdkImage *) g_object_ref (image);
 }
 
 /**
  * gdk_image_unref:
  * @image: a #GdkImage
- *
+ * 
  * Deprecated function; use g_object_unref() instead.
- *
- * Deprecated: 2.0: Use g_object_unref() instead.
+ * 
  **/
 void
 gdk_image_unref (GdkImage *image)
@@ -125,6 +119,7 @@ gdk_image_set_colormap (GdkImage       *image,
       image->colormap = colormap;
       g_object_ref (image->colormap);
     }
+    
 }
 
 /**
@@ -221,7 +216,7 @@ allocate_scratch_images (GdkScratchImageInfo *info,
 	  gint j;
 	  
 	  for (j = 0; j < i; j++)
-	    g_object_unref (info->static_image[j]);
+	    g_object_unref (info->static_image[i]);
 	  
 	  return FALSE;
 	}
@@ -230,25 +225,6 @@ allocate_scratch_images (GdkScratchImageInfo *info,
   return TRUE;
 }
 
-static void
-scratch_image_info_display_closed (GdkDisplay          *display,
-                                   gboolean             is_error,
-                                   GdkScratchImageInfo *image_info)
-{
-  gint i;
-
-  g_signal_handlers_disconnect_by_func (display,
-                                        scratch_image_info_display_closed,
-                                        image_info);
-
-  scratch_image_infos = g_slist_remove (scratch_image_infos, image_info);
-
-  for (i = 0; i < image_info->n_images; i++)
-    g_object_unref (image_info->static_image[i]);
-
-  g_free (image_info);
-}
-
 static GdkScratchImageInfo *
 scratch_image_info_for_depth (GdkScreen *screen,
 			      gint       depth)
@@ -272,10 +248,6 @@ scratch_image_info_for_depth (GdkScreen *screen,
   image_info->depth = depth;
   image_info->screen = screen;
 
-  g_signal_connect (gdk_screen_get_display (screen), "closed",
-                    G_CALLBACK (scratch_image_info_display_closed),
-                    image_info);
-
   /* Try to allocate as few possible shared images */
   for (i=0; i < G_N_ELEMENTS (possible_n_images); i++)
     {
@@ -450,6 +422,3 @@ gdk_image_new (GdkImageType  type,
   return _gdk_image_new_for_depth (gdk_visual_get_screen (visual), type,
 				   visual, width, height, -1);
 }
-
-#define __GDK_IMAGE_C__
-#include "gdkaliasdef.c"
