@@ -35,7 +35,9 @@
 #include <gtk/gtkwindow.h>
 #endif
 
-G_BEGIN_DECLS
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
 
 /* Priorities for redrawing and resizing
  */
@@ -55,6 +57,9 @@ G_BEGIN_DECLS
 
 #endif /* GTK_DISABLE_DEPRECATED */
 
+typedef void	(*GtkModuleInitFunc)        (gint	  *argc,
+				             gchar      ***argv);
+typedef void	(*GtkModuleDisplayInitFunc) (GdkDisplay   *display);
 typedef gint	(*GtkKeySnoopFunc)	    (GtkWidget	  *grab_widget,
 					     GdkEventKey  *event,
 					     gpointer	   func_data);
@@ -76,32 +81,19 @@ GTKMAIN_C_VAR const guint gtk_minor_version;
 GTKMAIN_C_VAR const guint gtk_micro_version;
 GTKMAIN_C_VAR const guint gtk_binary_age;
 GTKMAIN_C_VAR const guint gtk_interface_age;
-const gchar* gtk_check_version (guint	required_major,
-			        guint	required_minor,
-			        guint	required_micro);
+gchar* gtk_check_version (guint	required_major,
+			  guint	required_minor,
+			  guint	required_micro);
 
 
 /* Initialization, exit, mainloop and miscellaneous routines
  */
 
-gboolean gtk_parse_args           (int    *argc,
-				   char ***argv);
-
 void     gtk_init                 (int    *argc,
                                    char ***argv);
 
 gboolean gtk_init_check           (int    *argc,
                                    char ***argv);
-  
-gboolean gtk_init_with_args       (int            *argc,
-				   char         ***argv,
-				   char           *parameter_string,
-				   GOptionEntry   *entries,
-				   char           *translation_domain,
-				   GError        **error);
-
-GOptionGroup *gtk_get_option_group (gboolean open_default_display);
-  
 #ifdef G_PLATFORM_WIN32
 
 /* Variants that are used to check for correct struct packing
@@ -219,8 +211,15 @@ gboolean _gtk_boolean_handled_accumulator (GSignalInvocationHint *ihint,
                                    const GValue          *handler_return,
                                    gpointer               dummy);
 
+gchar * _gtk_find_module     (const gchar *name,
+			      const gchar *type);
+gchar **_gtk_get_module_path (const gchar *type);
+
 gchar *_gtk_get_lc_ctype (void);
 
-G_END_DECLS
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
 
 #endif /* __GTK_MAIN_H__ */
