@@ -1,4 +1,4 @@
-#include "config.h"
+#include <config.h>
 
 #include <string.h>
 #include <sys/types.h>
@@ -11,20 +11,18 @@
 #include <gtk/gtk.h>
 
 #ifdef G_OS_WIN32
-#  define WIN32_MEAN_AND_LEAN
-#  include <windows.h> /* ExtractAssociatedIcon */
 #  include <io.h>
-#  define localtime_r(t,b) localtime(t)
+#  define localtime_r(t,b) *(b) = *localtime (t)
 #  ifndef S_ISREG
 #    define S_ISREG(m) ((m) & _S_IFREG)
 #  endif
-#  include <gdk/win32/gdkwin32.h> /* gdk_win32_hdc_get */
 #endif
 
 #include "prop-editor.h"
 
 static GtkWidget *preview_label;
 static GtkWidget *preview_image;
+static GtkFileChooserAction action;
 
 static void
 print_current_folder (GtkFileChooser *chooser)
@@ -32,7 +30,7 @@ print_current_folder (GtkFileChooser *chooser)
   gchar *uri;
 
   uri = gtk_file_chooser_get_current_folder_uri (chooser);
-  g_print ("Current folder changed :\n  %s\n", uri);
+  g_print ("Current folder changed :\n  %s\n", uri ? uri : "(null)");
   g_free (uri);
 }
 
@@ -82,7 +80,7 @@ response_cb (GtkDialog *dialog,
     }
   else
     g_print ("Dialog was closed\n");
-  
+
   gtk_main_quit ();
 }
 
@@ -90,7 +88,7 @@ static gboolean
 no_backup_files_filter (const GtkFileFilterInfo *filter_info,
 			gpointer                 data)
 {
-  gsize len = strlen (filter_info->display_name);
+  gsize len = filter_info->display_name ? strlen (filter_info->display_name) : 0;
   if (len > 0 && filter_info->display_name[len - 1] == '~')
     return 0;
   else
@@ -138,20 +136,19 @@ static void
 size_prepared_cb (GdkPixbufLoader *loader, 
 		  int              width,
 		  int              height,
-		  gpointer         data)
+		  int             *data)
 {
-	struct {
-		int width;
-		int height;
-	} *info = data;
+	int des_width = data[0];
+	int des_height = data[1];
 
-	if ((double)height * (double)info->width >
-	    (double)width * (double)info->height) {
-		width = 0.5 + (double)width * (double)info->height / (double)height;
-		height = info->height;
+	if (des_height >= height && des_width >= width) {
+		/* Nothing */
+	} else if ((double)height * des_width > (double)width * des_height) {
+		width = 0.5 + (double)width * des_height / (double)height;
+		height = des_height;
 	} else {
-		height = 0.5 + (double)height * (double)info->width / (double)width;
-		width = info->width;
+		height = 0.5 + (double)height * des_width / (double)width;
+		width = des_width;
 	}
 
 	gdk_pixbuf_loader_set_size (loader, width, height);
@@ -165,10 +162,7 @@ my_new_from_file_at_size (const char *filename,
 {
 	GdkPixbufLoader *loader;
 	GdkPixbuf       *pixbuf;
-	struct {
-		int width;
-		int height;
-	} info;
+	int              info[2];
 	struct stat st;
 
 	guchar buffer [4096];
@@ -204,9 +198,9 @@ my_new_from_file_at_size (const char *filename,
 #ifdef DONT_PRESERVE_ASPECT
 	gdk_pixbuf_loader_set_size (loader, width, height);
 #else
-	info.width = width;
-	info.height = height;
-	g_signal_connect (loader, "size-prepared", G_CALLBACK (&size_prepared_cb), &info);
+	info[0] = width;
+	info[1] = height;
+	g_signal_connect (loader, "size-prepared", G_CALLBACK (size_prepared_cb), info);
 #endif	
 
 	while (!feof (f)) {
@@ -222,6 +216,7 @@ my_new_from_file_at_size (const char *filename,
 
 	fclose (f);
 
+	g_assert (*error == NULL);
 	if (!gdk_pixbuf_loader_close (loader, error)) {
 		g_object_unref (G_OBJECT (loader));
 		return NULL;
@@ -231,6 +226,11 @@ my_new_from_file_at_size (const char *filename,
 
 	if (!pixbuf) {
 		g_object_unref (G_OBJECT (loader));
+
+		/* did the loader set an error? */
+		if (*error != NULL)
+			return NULL;
+
 		g_set_error (error,
                              GDK_PIXBUF_ERROR,
                              GDK_PIXBUF_ERROR_FAILED,
@@ -294,11 +294,87 @@ update_preview_cb (GtkFileChooser *chooser)
 	}
       
       g_free (filename);
+
+      if (error)
+	g_error_free (error);
     }
 
   gtk_file_chooser_set_preview_widget_active (chooser, have_preview);
 }
 
+static void
+set_current_folder (GtkFileChooser *chooser,
+		    const char     *name)
+{
+  if (!gtk_file_chooser_set_current_folder (chooser, name))
+    {
+      GtkWidget *dialog;
+
+      dialog = gtk_message_dialog_new (GTK_WINDOW (chooser),
+				       GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+				       GTK_MESSAGE_ERROR,
+				       GTK_BUTTONS_CLOSE,
+				       "Could not set the folder to %s",
+				       name);
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+    }
+}
+
+static void
+set_folder_nonexistent_cb (GtkButton      *button,
+			   GtkFileChooser *chooser)
+{
+  set_current_folder (chooser, "/nonexistent");
+}
+
+static void
+set_folder_existing_nonexistent_cb (GtkButton      *button,
+				    GtkFileChooser *chooser)
+{
+  set_current_folder (chooser, "/usr/nonexistent");
+}
+
+static void
+set_filename (GtkFileChooser *chooser,
+	      const char     *name)
+{
+  if (!gtk_file_chooser_set_filename (chooser, name))
+    {
+      GtkWidget *dialog;
+
+      dialog = gtk_message_dialog_new (GTK_WINDOW (chooser),
+				       GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+				       GTK_MESSAGE_ERROR,
+				       GTK_BUTTONS_CLOSE,
+				       "Could not select %s",
+				       name);
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+    }
+}
+
+static void
+set_filename_nonexistent_cb (GtkButton      *button,
+			     GtkFileChooser *chooser)
+{
+  set_filename (chooser, "/nonexistent");
+}
+
+static void
+set_filename_existing_nonexistent_cb (GtkButton      *button,
+				      GtkFileChooser *chooser)
+{
+  set_filename (chooser, "/usr/nonexistent");
+}
+
+static void
+kill_dependent (GtkWindow *win, GtkObject *dep)
+{
+  gtk_object_destroy (dep);
+  g_object_unref (dep);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -307,21 +383,55 @@ main (int argc, char **argv)
   GtkWidget *button;
   GtkWidget *dialog;
   GtkWidget *prop_editor;
+  GtkWidget *extra;
   GtkFileFilter *filter;
   GtkWidget *preview_vbox;
-  GtkWidget *extra;
+  int i;
+  gboolean multiple = FALSE;
   
   gtk_init (&argc, &argv);
 
+  action = GTK_FILE_CHOOSER_ACTION_OPEN;
+
+  /* lame-o arg parsing */
+  for (i = 1; i < argc; i++)
+    {
+      if (! strcmp ("--action=open", argv[i]))
+	action = GTK_FILE_CHOOSER_ACTION_OPEN;
+      else if (! strcmp ("--action=save", argv[i]))
+	action = GTK_FILE_CHOOSER_ACTION_SAVE;
+      else if (! strcmp ("--action=select_folder", argv[i]))
+	action = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
+      else if (! strcmp ("--action=create_folder", argv[i]))
+	action = GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER;
+      else if (! strcmp ("--multiple", argv[i]))
+	multiple = TRUE;
+    }
+
   dialog = g_object_new (GTK_TYPE_FILE_CHOOSER_DIALOG,
-			 "action", GTK_FILE_CHOOSER_ACTION_OPEN,
-			 "title", "Select a file",
+			 "action", action,
+			 "file-system-backend", "gtk+",
+			 "select-multiple", multiple,
 			 NULL);
-			 
-  gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-			  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-			  GTK_STOCK_OPEN, GTK_RESPONSE_OK,
-			  NULL);
+  switch (action)
+    {
+    case GTK_FILE_CHOOSER_ACTION_OPEN:
+    case GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER:
+      gtk_window_set_title (GTK_WINDOW (dialog), "Select a file");
+      gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+			      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			      GTK_STOCK_OPEN, GTK_RESPONSE_OK,
+			      NULL);
+      break;
+    case GTK_FILE_CHOOSER_ACTION_SAVE:
+    case GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER:
+      gtk_window_set_title (GTK_WINDOW (dialog), "Save a file");
+      gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+			      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			      GTK_STOCK_SAVE, GTK_RESPONSE_OK,
+			      NULL);
+      break;
+    }
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
   
   g_signal_connect (dialog, "selection-changed",
@@ -354,8 +464,10 @@ main (int argc, char **argv)
   gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
 
   /* Preview widget */
+  /* THIS IS A TERRIBLE PREVIEW WIDGET, AND SHOULD NOT BE COPIED AT ALL.
+   */
   preview_vbox = gtk_vbox_new (0, FALSE);
-  gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (dialog), preview_vbox);
+  /*  gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (dialog), preview_vbox);*/
   
   preview_label = gtk_label_new (NULL);
   gtk_box_pack_start (GTK_BOX (preview_vbox), preview_label, TRUE, TRUE, 0);
@@ -370,7 +482,9 @@ main (int argc, char **argv)
 		    G_CALLBACK (update_preview_cb), NULL);
 
   /* Extra widget */
-  extra = gtk_check_button_new_with_mnemonic ("_Frobnicate the file");
+
+  extra = gtk_check_button_new_with_mnemonic ("Lar_t whoever asks about this button");
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (extra), TRUE);
   gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (dialog), extra);
 
   /* Shortcuts */
@@ -379,8 +493,6 @@ main (int argc, char **argv)
 					    "file:///usr/share/pixmaps",
 					    NULL);
 
-  /* Done with the dialog */
-  gtk_window_set_default_size (GTK_WINDOW (dialog), 600, 400);
   /* show_all() to reveal bugs in composite widget handling */
   gtk_widget_show_all (dialog);
 
@@ -394,6 +506,7 @@ main (int argc, char **argv)
   gtk_container_add (GTK_CONTAINER (control_window), vbbox);
 
   button = gtk_button_new_with_mnemonic ("_Select all");
+  gtk_widget_set_sensitive (button, multiple);
   gtk_container_add (GTK_CONTAINER (vbbox), button);
   g_signal_connect_swapped (button, "clicked",
 			    G_CALLBACK (gtk_file_chooser_select_all), dialog);
@@ -403,9 +516,39 @@ main (int argc, char **argv)
   g_signal_connect_swapped (button, "clicked",
 			    G_CALLBACK (gtk_file_chooser_unselect_all), dialog);
 
+  button = gtk_button_new_with_label ("set_current_folder (\"/nonexistent\")");
+  gtk_container_add (GTK_CONTAINER (vbbox), button);
+  g_signal_connect (button, "clicked",
+		    G_CALLBACK (set_folder_nonexistent_cb), dialog);
+
+  button = gtk_button_new_with_label ("set_current_folder (\"/usr/nonexistent\"");
+  gtk_container_add (GTK_CONTAINER (vbbox), button);
+  g_signal_connect (button, "clicked",
+		    G_CALLBACK (set_folder_existing_nonexistent_cb), dialog);
+
+  button = gtk_button_new_with_label ("set_filename (\"/nonexistent\"");
+  gtk_container_add (GTK_CONTAINER (vbbox), button);
+  g_signal_connect (button, "clicked",
+		    G_CALLBACK (set_filename_nonexistent_cb), dialog);
+
+  button = gtk_button_new_with_label ("set_filename (\"/usr/nonexistent\"");
+  gtk_container_add (GTK_CONTAINER (vbbox), button);
+  g_signal_connect (button, "clicked",
+		    G_CALLBACK (set_filename_existing_nonexistent_cb), dialog);
+
   gtk_widget_show_all (control_window);
-  
+
+  g_object_ref (control_window);
+  g_signal_connect (G_OBJECT (dialog), "destroy",
+		    G_CALLBACK (kill_dependent), control_window);
+
+  /* We need to hold a ref until we have destroyed the widgets, just in case
+   * someone else destroys them.  We explicitly destroy windows to catch leaks.
+   */
+  g_object_ref (dialog);
   gtk_main ();
+  gtk_widget_destroy (dialog);
+  g_object_unref (dialog);
 
   return 0;
 }
